/* 
 
Copyright 2006 Eric Hakenholz

This file is part of C.a.R. software.

    C.a.R. is a free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    C.a.R. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
 
 package eric;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import rene.gui.Global;

class JPaletteZone extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private rene.zirkel.ZirkelFrame ZF;
	JPaletteZoneTitleBar ZoneTitle;
	JPaletteZoneContent ZoneContent;
	JLabel label = null;
	JPalette JP;
	JPaletteManager JPM;
	String title;
	MouseAdapter TitleMouseAdapter;
	String name;
	private int x = -1;
	private int y = -1;
	private final Icon rightTriangle = new ImageIcon(getClass().getResource(
	"/eric/icons/palette/PaletteTriangleDroite.png"));
	private final Icon bottomTriangle = new ImageIcon(getClass().getResource(
	"/eric/icons/palette/PaletteTriangleBas.png"));
	boolean ContentVisible = true;
	int PW = 193;// Palette width
	public boolean mainmember = true;

	@Override
	public void paintComponent(final java.awt.Graphics g) {
		// super.paintComponent(g);
	}

	public JPaletteZone(final rene.zirkel.ZirkelFrame zf,
			final JPaletteManager jpm, final JPalette jp,
			final String PartTitle, final String myname) {
		init(zf, jpm, jp, PartTitle, myname);
	}

	// Constructor for dialogs other than MainPalette tools :
	public JPaletteZone(final rene.zirkel.ZirkelFrame zf,
			final JPaletteManager jpm, final JPalette jp,
			final String PartTitle, final String myname, final int w) {
		setWidth(w);
		init(zf, jpm, jp, PartTitle, myname);
		mainmember = false;
	}

	public void init(final rene.zirkel.ZirkelFrame zf,
			final JPaletteManager jpm, final JPalette jp,
			final String PartTitle, final String myname) {
		ZF = zf;
		JPM = jpm;
		JP = jp;
		title = PartTitle;
		name = myname;
		if (JPM != null) {
			PW = JPM.paletteiconsize * 6 + 1;
		}
		this.setLayout(new javax.swing.BoxLayout(this,
				javax.swing.BoxLayout.Y_AXIS));
		ZoneTitle = new JPaletteZoneTitleBar(JP, this, title);
		ZoneContent = new JPaletteZoneContent(zf, JP, this);
		this.setBackground(new java.awt.Color(204, 204, 204));
		this.add(ZoneTitle);
		this.add(ZoneContent);
	}

	private void fixsize(final JComponent cp, final int w, final int h) {
		final Dimension d = new Dimension(w, h);
		cp.setMaximumSize(d);
		cp.setMinimumSize(d);
		cp.setPreferredSize(d);
		cp.setSize(d);
	}

	public void setWidth(final int w) {
		PW = w;
	}

	// must only be called by the jpalettemanager constructor :
	public void CollapseOrExpand() {
		if (Global.getParameter("hidepalette." + name, true)) {
			collapse();
		}
	}

	public void collapse() {
		Global.setParameter("hidepalette." + name, true);
		ContentVisible = false;
		ZoneTitle.title.setIcon(rightTriangle);
		JP.validate();
		this.remove(1);
		JP.validate();
		JP.pack();
		JP.Content.paintImmediately(0, 0, PW,
				JP.Content.getPreferredSize().height);
	}

	public void expand() {
		Global.setParameter("hidepalette." + name, false);
		ContentVisible = true;
		ZoneTitle.title.setIcon(bottomTriangle);
		this.add(ZoneContent);
		if (JPM != null) {
			JPM.CollapseToFitScreenHeight(this);
		}
        
		JP.validate();
		JP.pack();
		JP.Content.paintImmediately(0, 0, PW,
				JP.Content.getPreferredSize().height);
	}

	public void addCursor(final JCursor mycursor) {
		ZoneContent.add(mycursor);
	}

	public void addColorPicker(final JColorPanel mycolorpanel) {
		ZoneContent.add(mycolorpanel);
	}

	public JLabel addLabel(final String mytxt) {
		final JLabel myLabel = new JLabel(mytxt);
		myLabel.setOpaque(false);
		myLabel.setFont(new java.awt.Font(JGlobals.GlobalFont, 1, 11));
		myLabel.setForeground(new Color(100, 100, 100));
		myLabel.setHorizontalAlignment(SwingConstants.CENTER);
		final JPanel myline = ZoneContent.newLabelLine();
		fixsize(myLabel, myline.getSize().width, myline.getSize().height);
		myline.add(myLabel);
		addNewLine();
		return myLabel;
	}

	private JPanel add7iconsString(final String myname[], final Vector Group) {
		JPanel myLastLine = null;
		for (final String element : myname) {
			if (JPM.acceptedIcon(element)) {
				final JIcon myicon = new JIcon(ZF, JPM, this, element, Group);
				if (JPM.MustBeFirstIconOnLine(element)) {
					addNewLine();
				}

				fixsize(myicon, JPM.paletteiconsize - 5,
						JPM.paletteiconsize - 5);
				myLastLine = getLastLine();
				myLastLine.add(myicon);
				JPM.AllIcons.add(myicon);
			}
		}
		return myLastLine;
	}

	public void addPointIcons(final String myname[], final Vector Group) {
		JPM.MW.PointLabel.addPaletteJLabel(add7iconsString(myname, Group));
	}

	public void addColorIcons(final String myname[], final Vector Group) {
		final JPanel myLine = add7iconsString(myname, Group);
		myLine.add(JColorPicker.margin(5));

		JPM.MW.ColorPicker = new JColorPicker(JPM.paletteiconsize - 5, 6, 3,
				Group) {
			/**
					 * 
					 */
					private static final long serialVersionUID = 1L;

			@Override
			public void doChange() {
				JPM.setObjectColor(getCurrentColor());
			}

			@Override
			public void afterSelect() {
				JPM.setObjectColor(getCurrentColor());
				JPM.MW.ZF.setinfo("prop_scolor", false);
			}

			@Override
			public void setPalettes() {
				setUsedColors(JPM.MW.ZF.ZC.getConstruction().getSpecialColors());
			}
		};
		myLine.add(JPM.MW.ColorPicker);
		addNewLine();
	}

	public JIcon addIcon(String name, final Vector Group) {
		boolean invisible = false;
		if (name.startsWith("@@")) {
			invisible = true;
			name = name.substring(2);
		}
		if (JPM.acceptedIcon(name)) {
			final JIcon myicon = new JIcon(ZF, JPM, this, name, Group);
			if (invisible) {
				JPM.AllIcons.add(myicon);
				return myicon;
			}
			if (JPM.MustBeFirstIconOnLine(name)) {
				addNewLine();
			}
			getLastLine().add(myicon);
			JPM.AllIcons.add(myicon);
			return myicon;
		}
		return null;
	}

	public void addIcons(final String myname[], final Vector Group) {
		for (final String element : myname) {
			addIcon(element, Group);
			// if (JPM.acceptedIcon(myname[i])) {
			// JIcon myicon=new JIcon(ZF,JPM,this,myname[i],Group);
			// if (JPM.MustBeFirstIconOnLine(myname[i])) addNewLine();
			// getLastLine().add(myicon);
			// JPM.AllIcons.add(myicon);
			// }
		}
		
	}

	public void addVirtualIcons(final String myname[], final Vector Group) {
		for (final String element : myname) {
			final JIcon myicon = new JIcon(ZF, JPM, this, element, Group);
			JPM.AllIcons.add(myicon);
		}
		
	}

	public JIcon addToggleIcon(final String myname) {
		if (JPM.acceptedIcon(myname)) {
			final JIcon myicon = new JIcon(ZF, JPM, this, myname, new Vector());
			if (JPM.MustBeFirstIconOnLine(myname)) {
				addNewLine();
			}
			getLastLine().add(myicon);
			JPM.AllIcons.add(myicon);
			return myicon;
		} else {
			return null;
		}
	}

	public void addSimpleIcon(final String myname) {
		if (JPM.acceptedIcon(myname)) {
			final JIcon myicon = new JIcon(ZF, JPM, this, myname, null);
			if (JPM.MustBeFirstIconOnLine(myname)) {
				addNewLine();
			}
			getLastLine().add(myicon);
			JPM.AllIcons.add(myicon);
		}
		
	}

	public void addNewLine() {
		ZoneContent.newline();
	}

	public JPanel getLastLine() {
		JPanel lastline;
		if (ZoneContent.getComponentCount() == 0) {
			lastline = ZoneContent.newline();
		} else {
			lastline = (JPanel) ZoneContent.getComponent(ZoneContent
					.getComponentCount() - 1);
			if (lastline.getComponentCount() == 6) {
				ZoneContent.add(lastline);
				lastline = ZoneContent.newline();
			}
		}
		return lastline;
	}

	public void setTitle(final String tt) {
		ZoneTitle.title.setText(tt);
	}

	

	private class JPaletteZoneTitleBar extends JPanel {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		JLabel title = new JLabel();
		private final JPalette JP;
		private final JPaletteZone myZone;
		private boolean MouseOn = false;
		JPalette FP;

		@Override
		public void paintComponent(final java.awt.Graphics g) {
			final Image OnImage = (JPM == null) ? JMacrosTools.CurrentJZF.JZT
					.getImage("PaletteTitleBarH.png") : JPM.MW.JZT
					.getImage("PaletteTitleBarH.png");
					final Image OffImage = (JPM == null) ? JMacrosTools.CurrentJZF.JZT
							.getImage("PaletteTitleBarN.png") : JPM.MW.JZT
							.getImage("PaletteTitleBarN.png");
							final java.awt.Dimension d = this.getSize();
							if (MouseOn) {
								g.drawImage(OnImage, 0, 0, PW, d.height, this);
							} else {
								g.drawImage(OffImage, 0, 0, PW, d.height, this);
							}
							
		}

		public JPaletteZoneTitleBar(final JPalette jp, final JPaletteZone myz,
				final String TitleBar) {
			JP = jp;
			myZone = myz;
			this.setLayout(new javax.swing.BoxLayout(this,
					javax.swing.BoxLayout.X_AXIS));
			this.setFocusable(false);
			this.setAlignmentX(0);
			this.setMaximumSize(new java.awt.Dimension(PW, 17));
			this.setMinimumSize(new java.awt.Dimension(PW, 17));
			this.setPreferredSize(new java.awt.Dimension(PW, 17));
			this.setSize(PW, 17);
			title.setText(TitleBar);
			title.setIcon(bottomTriangle);
			title.setIconTextGap(7);
			title.setFont(new java.awt.Font(JGlobals.GlobalFont, 0, 11));
			title.setHorizontalAlignment(SwingConstants.LEFT);
			title.setMaximumSize(new java.awt.Dimension(PW - 17, 17));
			title.setMinimumSize(new java.awt.Dimension(PW - 17, 17));
			title.setPreferredSize(new java.awt.Dimension(PW - 17, 17));
			this.add(title, null);

			TitleMouseAdapter = new MouseAdapter() {

				@Override
				public void mouseReleased(final MouseEvent e) {
					MouseOn = false;
					ZoneTitle.repaint();
					x = e.getX();
					y = e.getY();
				}

				@Override
				public void mousePressed(final MouseEvent e) {
					MouseOn = true;
					ZoneTitle.repaint();
					x = e.getX();
					y = e.getY();
				}

				@Override
				public void mouseClicked(final MouseEvent e) {
					if (ContentVisible) {

						myZone.collapse();
					} else {
						myZone.expand();
					}
				}
			};

			final JButton mybtn = new JButton();
			mybtn.setOpaque(false);
			mybtn.setContentAreaFilled(false);
			if (JP.MainPalette) {
				mybtn.setIcon(new javax.swing.ImageIcon(getClass().getResource(
				"/eric/icons/palette/Pdetach.gif")));

				mybtn.addMouseListener(new MouseAdapter() {

					@Override
					public void mousePressed(final MouseEvent e) {
						MouseOn = true;
						ZoneTitle.repaint();
						FP = JPM.detachpalette1(myZone);
					}

					@Override
					public void mouseReleased(final MouseEvent e) {
						JPM.detachpalette2(FP, myZone);
					}
				});
			} else {
				mybtn.setIcon(new javax.swing.ImageIcon(getClass().getResource(
				"/eric/icons/palette/Pclose.png")));
				mybtn.setRolloverIcon(new javax.swing.ImageIcon(getClass()
						.getResource("/eric/icons/palette/Pcloseover.png")));

				mybtn.addMouseListener(new MouseAdapter() {

					@Override
					public void mousePressed(final MouseEvent e) {
						MouseOn = true;
						ZoneTitle.repaint();
						if (myZone.mainmember) {
							JPM.dockpalette(JP, myZone);
						} else {
							MouseOn = false;
							myZone.JP.setVisible(false);

						}

					}
				});
			}
			

			mybtn.setOpaque(false);
			mybtn.setBorder(BorderFactory.createEmptyBorder());
			// this.add(mybtn, null);
			if ((JPM == null) || (JGlobalPreferences.undockpalette)) {
				this.add(mybtn, null);
			}
			this.addMouseListener(TitleMouseAdapter);
			this.addMouseMotionListener(new MouseMotionAdapter() {

				@Override
				public void mouseDragged(final MouseEvent e) {
                    if (JP.MainPalette) return;
					final int xx = JP.getLocation().x;
					final int yy = JP.getLocation().y;
					final int Ybottom = yy + (e.getY() - y)
					+ JP.getPreferredSize().height;
					final int ScreenBottom = Toolkit.getDefaultToolkit()
					.getScreenSize().height;
					if (Ybottom < ScreenBottom) {
						JP
						.setLocation(xx + (e.getX() - x), yy
								+ (e.getY() - y));
					} else {
						JP.setLocation(xx + (e.getX() - x), ScreenBottom
								- JP.getPreferredSize().height);
					}

				}
			});
		}
	}

	public class JPaletteZoneContent extends javax.swing.JPanel {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		@Override
		public void paintComponent(final java.awt.Graphics g) {
			// super.paintComponent(g);
		}

		public JPaletteZoneContent(final rene.zirkel.ZirkelFrame zf,
				final JPalette jp, final JPaletteZone myz) {
			initComponents();
		}

		public void initComponents() {
			this.setLayout(new javax.swing.BoxLayout(this,
					javax.swing.BoxLayout.Y_AXIS));
		}

		public JPanel newline() {
			final JPanel myline = new ContentLine();
			this.add(myline);
			return myline;
		}

		public JPanel newLabelLine() {
			final JPanel myline = new LabelLine();
			this.add(myline);
			return myline;
		}

		class ContentLine extends javax.swing.JPanel {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			@Override
			public void paintComponent(final java.awt.Graphics g) {
				final java.awt.Dimension d = this.getSize();
				g.drawImage(JPM.MW.JZT.getImage("palbackground.gif"), 0, 0,
						d.width, d.height, this);
			}

			public ContentLine() {
				final int lineheight = (JPM == null) ? 32 : JPM.paletteiconsize;
				this.setLayout(new javax.swing.BoxLayout(this,
						javax.swing.BoxLayout.X_AXIS));
				this.setAlignmentX(0);
				fixsize(this, PW, lineheight);
				// this.setMaximumSize(new java.awt.Dimension(PW, lineheight));
				// this.setMinimumSize(new java.awt.Dimension(PW, lineheight));
				// this.setPreferredSize(new java.awt.Dimension(PW,
				// lineheight));
				// this.setSize(PW,lineheight);
			}
		}

		class LabelLine extends javax.swing.JPanel {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			@Override
			public void paintComponent(final java.awt.Graphics g) {
				final java.awt.Dimension d = this.getSize();
				g.drawImage(JPM.MW.JZT.getImage("palbackground.gif"), 0, 0,
						d.width, d.height, this);
				final Graphics2D g2 = (Graphics2D) g;
				g2.setComposite(AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, 0.05f));
				g2.setColor(new Color(0, 0, 255));
				g2.fillRect(3, 3, d.width - 6, d.height - 6);
				g2.setComposite(AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, 0.15f));
				g2.drawRect(3, 3, d.width - 7, d.height - 7);
				g2.setComposite(AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, 1f));
			}

			public LabelLine() {
				final int lineheight = 20;
				this.setLayout(new javax.swing.BoxLayout(this,
						javax.swing.BoxLayout.X_AXIS));
				this.setAlignmentX(0);
				fixsize(this, PW, lineheight);
			}
		}
	}
}
